@@ -1,6 +1,5 @@
 # encoding: UTF-8
 require 'spec_helper'
-require 'stringio'
 
 RSpec.describe Mysql2::Client do
   context "using defaults file" do
@@ -34,6 +33,12 @@ RSpec.describe Mysql2::Client do
     }.to raise_error(Mysql2::Error)
   end
 
+  it "should raise an exception on non-string encodings" do
+    expect {
+      Mysql2::Client.new(DatabaseCredentials['root'].merge(:encoding => :fake))
+    }.to raise_error(TypeError)
+  end
+
   it "should not raise an exception on create for a valid encoding" do
     expect {
       Mysql2::Client.new(DatabaseCredentials['root'].merge(:encoding => "utf8"))
@@ -200,22 +205,21 @@ RSpec.describe Mysql2::Client do
 
     if RUBY_PLATFORM =~ /mingw|mswin/
       it "cannot be disabled" do
-        stderr, $stderr = $stderr, StringIO.new
+        expect do
+          client = Mysql2::Client.new(DatabaseCredentials['root'].merge(:automatic_close => false))
+          expect(client.automatic_close?).to be(true)
+        end.to output(/always closed by garbage collector/).to_stderr
 
-        begin
-          Mysql2::Client.new(DatabaseCredentials['root'].merge(:automatic_close => false))
-          expect($stderr.string).to include('always closed by garbage collector')
-          $stderr.reopen
+        expect do
+          client = Mysql2::Client.new(DatabaseCredentials['root'].merge(:automatic_close => true))
+          expect(client.automatic_close?).to be(true)
+        end.to_not output(/always closed by garbage collector/).to_stderr
 
-          client = Mysql2::Client.new(DatabaseCredentials['root'])
+        expect do
+          client = Mysql2::Client.new(DatabaseCredentials['root'].merge(:automatic_close => true))
           client.automatic_close = false
-          expect($stderr.string).to include('always closed by garbage collector')
-          $stderr.reopen
-
-          expect { client.automatic_close = true }.to_not change { $stderr.string }
-        ensure
-          $stderr = stderr
-        end
+          expect(client.automatic_close?).to be(true)
+        end.to output(/always closed by garbage collector/).to_stderr
       end
     else
       it "can be configured" do
@@ -263,13 +267,14 @@ RSpec.describe Mysql2::Client do
   end
 
   it "should be able to connect to database with numeric-only name" do
-    creds = DatabaseCredentials['numericuser']
-    @client.query "CREATE DATABASE IF NOT EXISTS `#{creds['database']}`"
-    @client.query "GRANT ALL ON `#{creds['database']}`.* TO #{creds['username']}@`#{creds['host']}`"
+    database = 1235
+    @client.query "CREATE DATABASE IF NOT EXISTS `#{database}`"
 
-    expect { Mysql2::Client.new(creds) }.not_to raise_error
+    expect {
+      Mysql2::Client.new(DatabaseCredentials['root'].merge('database' => database))
+    }.not_to raise_error
 
-    @client.query "DROP DATABASE IF EXISTS `#{creds['database']}`"
+    @client.query "DROP DATABASE IF EXISTS `#{database}`"
   end
 
   it "should respond to #close" do
@@ -382,24 +387,24 @@ RSpec.describe Mysql2::Client do
 
   it "should expect connect_timeout to be a positive integer" do
     expect {
-      Mysql2::Client.new(:connect_timeout => -1)
+      Mysql2::Client.new(DatabaseCredentials['root'].merge(:connect_timeout => -1))
     }.to raise_error(Mysql2::Error)
   end
 
   it "should expect read_timeout to be a positive integer" do
     expect {
-      Mysql2::Client.new(:read_timeout => -1)
+      Mysql2::Client.new(DatabaseCredentials['root'].merge(:read_timeout => -1))
     }.to raise_error(Mysql2::Error)
   end
 
   it "should expect write_timeout to be a positive integer" do
     expect {
-      Mysql2::Client.new(:write_timeout => -1)
+      Mysql2::Client.new(DatabaseCredentials['root'].merge(:write_timeout => -1))
     }.to raise_error(Mysql2::Error)
   end
 
   it "should allow nil read_timeout" do
-    client = Mysql2::Client.new(:read_timeout => nil)
+    client = Mysql2::Client.new(DatabaseCredentials['root'].merge(:read_timeout => nil))
 
     expect(client.read_timeout).to be_nil
   end
@@ -559,7 +564,7 @@ RSpec.describe Mysql2::Client do
       context 'when a non-standard exception class is raised' do
         it "should close the connection when an exception is raised" do
           expect { Timeout.timeout(0.1, ArgumentError) { @client.query('SELECT SLEEP(1)') } }.to raise_error(ArgumentError)
-          expect { @client.query('SELECT 1') }.to raise_error(Mysql2::Error, 'closed MySQL connection')
+          expect { @client.query('SELECT 1') }.to raise_error(Mysql2::Error, 'MySQL client is not connected')
         end
 
         it "should handle Timeouts without leaving the connection hanging if reconnect is true" do
