@@ -6,11 +6,13 @@ RSpec.describe Mysql2::Statement do
     @client = Mysql2::Client.new(DatabaseCredentials['root'].merge(:encoding => "utf8"))
   end
 
+  def stmt_count
+    @client.query("SHOW STATUS LIKE 'Prepared_stmt_count'").first['Value'].to_i
+  end
+
   it "should create a statement" do
     statement = nil
-    expect { statement = @client.prepare 'SELECT 1' }.to change {
-      @client.query("SHOW STATUS LIKE 'Prepared_stmt_count'").first['Value'].to_i
-    }.by(1)
+    expect { statement = @client.prepare 'SELECT 1' }.to change(&method(:stmt_count)).by(1)
     expect(statement).to be_an_instance_of(Mysql2::Statement)
   end
 
@@ -59,6 +61,26 @@ RSpec.describe Mysql2::Statement do
     expect(rows).to eq([{ "1" => 1 }])
   end
 
+  it "should handle bignum but in int64_t" do
+    stmt = @client.prepare('SELECT ? AS max, ? AS min')
+    int64_max = (1 << 63) - 1
+    int64_min = -(1 << 63)
+    result = stmt.execute(int64_max, int64_min)
+    expect(result.to_a).to eq(['max' => int64_max, 'min' => int64_min])
+  end
+
+  it "should handle bignum but beyond int64_t" do
+    stmt = @client.prepare('SELECT ? AS max1, ? AS max2, ? AS max3, ? AS min1, ? AS min2, ? AS min3')
+    int64_max1 = (1 << 63)
+    int64_max2 = (1 << 64) - 1
+    int64_max3 = 1 << 64
+    int64_min1 = -(1 << 63) - 1
+    int64_min2 = -(1 << 64) + 1
+    int64_min3 = -0xC000000000000000
+    result = stmt.execute(int64_max1, int64_max2, int64_max3, int64_min1, int64_min2, int64_min3)
+    expect(result.to_a).to eq(['max1' => int64_max1, 'max2' => int64_max2, 'max3' => int64_max3, 'min1' => int64_min1, 'min2' => int64_min2, 'min3' => int64_min3])
+  end
+
   it "should keep its result after other query" do
     @client.query 'USE test'
     @client.query 'CREATE TABLE IF NOT EXISTS mysql2_stmt_q(a int)'
@@ -108,6 +130,35 @@ RSpec.describe Mysql2::Statement do
     expect(result.first.first[1]).to be_an_instance_of(Time)
   end
 
+  it "should prepare Date values" do
+    now = Date.today
+    statement = @client.prepare('SELECT ? AS a')
+    result = statement.execute(now)
+    expect(result.first['a'].to_s).to eql(now.strftime('%F'))
+  end
+
+  it "should prepare Time values with microseconds" do
+    now = Time.now
+    statement = @client.prepare('SELECT ? AS a')
+    result = statement.execute(now)
+    if RUBY_VERSION =~ /1.8/
+      expect(result.first['a'].strftime('%F %T %z')).to eql(now.strftime('%F %T %z'))
+    else
+      expect(result.first['a'].strftime('%F %T.%6N %z')).to eql(now.strftime('%F %T.%6N %z'))
+    end
+  end
+
+  it "should prepare DateTime values with microseconds" do
+    now = DateTime.now
+    statement = @client.prepare('SELECT ? AS a')
+    result = statement.execute(now)
+    if RUBY_VERSION =~ /1.8/
+      expect(result.first['a'].strftime('%F %T %z')).to eql(now.strftime('%F %T %z'))
+    else
+      expect(result.first['a'].strftime('%F %T.%6N %z')).to eql(now.strftime('%F %T.%6N %z'))
+    end
+  end
+
   it "should tell us about the fields" do
     statement = @client.prepare 'SELECT 1 as foo, 2'
     statement.execute
@@ -117,6 +168,13 @@ RSpec.describe Mysql2::Statement do
     expect(list[1]).to eq('2')
   end
 
+  it "should handle as a decimal binding a BigDecimal" do
+    stmt = @client.prepare('SELECT ? AS decimal_test')
+    test_result = stmt.execute(BigDecimal.new("123.45")).first
+    expect(test_result['decimal_test']).to be_an_instance_of(BigDecimal)
+    expect(test_result['decimal_test']).to eql(123.45)
+  end
+
   it "should update a DECIMAL value passing a BigDecimal" do
     @client.query 'USE test'
     @client.query 'DROP TABLE IF EXISTS mysql2_stmt_decimal_test'
@@ -128,6 +186,14 @@ RSpec.describe Mysql2::Statement do
     expect(test_result['decimal_test']).to eql(123.45)
   end
 
+  it "should warn but still work if cache_rows is set to false" do
+    @client.query_options.merge!(:cache_rows => false)
+    statement = @client.prepare 'SELECT 1'
+    result = nil
+    expect { result = statement.execute.to_a }.to output(/:cache_rows is forced for prepared statements/).to_stderr
+    expect(result.length).to eq(1)
+  end
+
   context "utf8_db" do
     before(:each) do
       @client.query("DROP DATABASE IF EXISTS test_mysql2_stmt_utf8")
@@ -681,9 +747,7 @@ RSpec.describe Mysql2::Statement do
   context 'close' do
     it 'should free server resources' do
       stmt = @client.prepare 'SELECT 1'
-      expect { stmt.close }.to change {
-        @client.query("SHOW STATUS LIKE 'Prepared_stmt_count'").first['Value'].to_i
-      }.by(-1)
+      expect { stmt.close }.to change(&method(:stmt_count)).by(-1)
     end
 
     it 'should raise an error on subsequent execution' do
